/*Copyright (C) 2019 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana

This file is part of Plataforma Integrada MEC.

Plataforma Integrada MEC is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Plataforma Integrada MEC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with Plataforma Integrada MEC.  If not, see <http://www.gnu.org/licenses/>.*/

import React, { useState, useEffect } from 'react'
import Card from '@material-ui/core/Card';
import styled from 'styled-components'
import Grid from '@material-ui/core/Grid';
import LoadingSpinner from '../../LoadingSpinner.js'
import PanelTemplateColecao from '../PanelComponents/TemplateColecao.js'
import { WhiteContainer, StyledGrid } from '../StyledComponents.js'
import CreateNewFolderIcon from '@material-ui/icons/CreateNewFolder';
import Title from '../PanelComponents/PanelTitle.js'
import CollectionCardFunction from '../../CollectionCardFunction.js'
import { ButtonsAreaColecao } from '../PanelComponents/ButtonsArea'
import CriarColecaoModal from '../../CriarColecaoModal.js'
import { fetchAllRequest, getRequest } from '../../HelperFunctions/getAxiosConfig'

//Image Import
import ImportImages from "ImportImages.js"
const ColecaoVazia = ImportImages('ColecaoVazia');

export default function TabPanelColecoes(props) {
  const [loading, handleLoading] = useState(true)

  const [errorInUserColl, setErrorInUserColl] = useState(false)
  const [errorInFollowedColl, setErrorInFollowedColl] = useState(false)

  const [userCollections, setUserCollections] = useState([])
  const [followedCollections, setFollowedCollections] = useState([])

  const [currLimitUserColl, setCurrLimitUserColl] = useState(4)
  const [currLimitFollowedColl, setCurrLimitFollowedColl] = useState(4)

  const [loadingMoreUserColl, setLoadingMoreUserColl] = useState(false);
  const [loadingMoreFollowedColl, setLoadingMoreFollowedColl] = useState(false);

  const [endOfUserColl, setEndOfUserColl] = useState(0);
  const [endOfFollowedColl, setEndOfFollowedColl] = useState(0);

  const removeColl = (itemId) => {
    let newSlice = userCollections.filter(item => item.id !== itemId);
    setUserCollections(newSlice);
    setEndOfUserColl(String(endOfUserColl - 1))
  };

  function handleSuccess(responseArr, headersArr) {
    setErrorInUserColl(responseArr[0].errors ? true : false)
    setErrorInFollowedColl(responseArr[1].errors ? true : false)

    handleLoading(false)
    setUserCollections(responseArr[0])
    setFollowedCollections(responseArr[1])

    if (headersArr[0].has('X-Total-Count')) {
      setEndOfUserColl(headersArr[0].get('X-Total-Count'));
    }
    if (headersArr[1].has('X-Total-Count')) {
      setEndOfFollowedColl(headersArr[1].get('X-Total-Count'));
    }
  }

  function handleError(error) {
    handleLoading(false)
    setErrorInFollowedColl(true)
    setErrorInUserColl(true)
  }

  const getInfo = () => {
    const urls = [
      `/users/${props.id}/collections?offset=0&limit=4`,
      `/users/${props.id}/following/Collection?offset=0&limit=4`
    ]
    fetchAllRequest(urls, handleSuccess, handleError)
  }

  useEffect(() => {
    handleLoading(true)
    getInfo()
  }, [])

  const showMoreUserCollections = (limite) => {
    const limit = limite;
    setLoadingMoreUserColl(true);
    setCurrLimitUserColl(currLimitUserColl + limit)
    const url = `/users/${props.id}/collections?offset=${currLimitUserColl}&limit=${limit}`;
    getRequest(url,
      (data) => {
        if (data.errors) {
          setLoadingMoreUserColl(false);
          setErrorInUserColl(true)
        }
        else if (data.length >= 1) {
          let currData = [...userCollections];
          currData = [...currData.concat(data)];
          setLoadingMoreUserColl(false);
          setUserCollections(currData);
        }
        else {
          setLoadingMoreUserColl(false);
        }
      },
      (error) => {
        setLoadingMoreUserColl(false);
        setErrorInUserColl(true)
      }
    )
  }

  const showMoreFollowedCollections = (limite) => {
    const limit = limite;
    setLoadingMoreFollowedColl(true);
    setCurrLimitFollowedColl(currLimitFollowedColl + limit)
    const url = `/users/${props.id}/following/Collection?offset=${currLimitFollowedColl}&limit=${limit}`;
    getRequest(url,
      (data) => {
        if (data.errors) {
          setLoadingMoreFollowedColl(false);
          setErrorInFollowedColl(true)
        }
        else if (data.length >= 1) {
          let currData = [...followedCollections];
          currData = [...currData.concat(data)];
          setLoadingMoreFollowedColl(false);
          setFollowedCollections(currData);
        }
        else {
          setLoadingMoreFollowedColl(false);
        }
      },
      (error) => {
        setLoadingMoreFollowedColl(false);
        setErrorInFollowedColl(true)
      }
    )
  }

  return (
    <>
      {
        loading ?
          (
            <LoadingSpinner text={'CARREGANDO COLEÇÕES'} />
          )
          :
          (
            [
              <React.Fragment>
                <Tentativa
                  title={"Minhas Coleções"}
                  length={userCollections.length}
                  noContentText={
                    <Grid direction='column' justify='center' alignItems='center'>
                      <Grid item>
                        <img src={ColecaoVazia} alt="ColecaoVazia" style={{ height: "150px", width: "150px", verticalAlign: "middle", border: "0" }} />
                      </Grid>
                      <Grid>
                        <span style={{ fontFamily: "Roboto", fontWeight: "lighter", fontSize: "24px", textAlign: 'center' }}>
                          Criamos a sua primeira Coleção!
                        </span>
                      </Grid>
                      <Grid>
                        <p style={{ fontFamily: "Roboto", fontSize: "16px", margin: "10px 0 0", fontWeight: "normal" }}>
                          Adicione nela recursos que você queira acessar mais tarde.
                          <br />
                          Crie novas coleções clicando no cartão roxo "Criar Colecão".
                        </p>
                      </Grid>
                    </Grid>
                  }
                  sliceArr={userCollections}
                  showMore={showMoreUserCollections}
                  loadingMore={loadingMoreUserColl}
                  end={endOfUserColl}
                  callback={getInfo}
                  error={errorInUserColl}
                  removeColl={removeColl}
                />

                <PanelTemplateColecao
                  title={"Coleções que você segue"}
                  length={followedCollections.length}
                  noContentText={"Você ainda não segue nenhuma coleção."}
                  sliceArr={followedCollections}
                  showMore={showMoreFollowedCollections}
                  loadingMore={loadingMoreFollowedColl}
                  end={endOfFollowedColl}
                  followed={true}
                  error={errorInFollowedColl}
                />
              </React.Fragment>
            ]
          )
      }
    </>
  )
}

function Tentativa(props) {
  const [modalOpen, toggleModal] = useState(false)
  const handleModal = () => { toggleModal(!modalOpen) };

  if (props.error)
    return (
      <WhiteContainer>
        <Title
          title={props.title}
          length={props.length}
        />
        <p
          style={{ fontSize: "15px", fontWeight: "lighter", margin: "0 0 10px", display: "flex", justifyContent: "center", textAlign: "center" }}
        >
          Erro ao tentar obter os dados
                </p>
      </WhiteContainer>
    )
  else
    return (
      <WhiteContainer>
        <CriarColecaoModal open={modalOpen} handleClose={() => { handleModal(); props.callback() }} />

        <Title
          title={props.title}
          length={props.end}
        />

        <StyledGrid container spacing={1} style={{ paddingLeft: "30px", paddingRight: "15px" }}>
          <Grid item xs={12} sm={6} md={'auto'} lg={3}>
            <CardDiv onClick={() => { handleModal() }}>
              <div style={{ backgroundColor: "#673ab7", display: "flex", height: "100%", width: "100%", justifyContent: "center", alignItems: "center", cursor: "pointer" }}>
                <CreateNewFolderIcon style={{ color: "#fff", fontSize: "70px" }} />
                <p style={{ fontSize: "16px", margin: "0 0 10px", color: "#fff" }}>
                  CRIAR COLEÇÃO
                            </p>
              </div>
            </CardDiv>
          </Grid>

          {
            props.length === 0 ?
              (
                [
                  <Grid item lg={6} md={4} sm={6} xs={12}>
                    <div>
                      {props.noContentText}
                    </div>
                  </Grid>
                ]
              )
              :
              (
                [
                  <React.Fragment>
                    {
                      props.sliceArr.map((card) =>
                        <Grid item xs={12} sm={6} md={'auto'} lg={3} key={card.id}>
                          <CollectionCardFunction
                            name={card.name}
                            tags={card.tags}
                            rating={card.review_average}
                            id={card.id}
                            author={card.owner.name}
                            description={card.description}
                            thumbnails={card.items_thumbnails}
                            avatar={card.owner.avatar}
                            likeCount={card.likes_count}
                            followed={card.followed}
                            liked={card.liked}
                            collections={card.collection_items}
                            authorID={card.owner.id}
                            removeColl={props.removeColl}
                          />
                        </Grid>
                      )
                    }
                  </React.Fragment>
                ]
              )
          }
        </StyledGrid>
        {
          props.loadingMore ?
            <LoadingSpinner text={'Carregando Recursos...'} />
            :
            <ButtonsAreaColecao
              sliceLength={props.sliceArr.length}
              length={props.length}
              showMore={props.showMore}
              total={props.end}
              end={String(props.sliceArr.length) === props.end}
            />
        }
      </WhiteContainer>
    )
}

const CardDiv = styled(Card)`
    margin-top : 10px;
    margin-bottom : 10px;
    height : 381px;
    width : 272.5px;
    ${'' /* float : left; */}
`
